@@ -93,7 +93,7 @@ typedef enum {
 
     fe_program_dump,
     fe_program_gnutar,
-    fe_program_backup_api,		/* require fe_sendsize_req_options */
+    fe_program_application_api,		/* require fe_sendsize_req_options */
 
     fe_options_compress_fast,
     fe_options_compress_best,
@@ -172,6 +172,13 @@ typedef enum {
     fe_req_options_config,
 
     fe_rep_sendsize_quoted_error,
+    fe_req_xml,
+    fe_pp_script,	// only in XML
+    fe_amindexd_DLE,
+    fe_amrecover_dle_in_header,
+    fe_xml_estimate,
+    fe_xml_property_priority,
+    fe_sendsize_rep_warning,
 
     /*
      * All new features must be inserted immediately *before* this entry.
