@@ -1835,6 +1835,6 @@ void
 yyerror(
     char *	s)
 {
-  printf("%s\n", s);
+  g_printf("%s\n", s);
 }
 
