@@ -1,6 +1,6 @@
 #!@PERL@
 #
-# Copyright (c) 2006 Zmanda Inc.  All Rights Reserved.
+# Copyright (c) 2005-2008 Zmanda Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published
@@ -15,8 +15,8 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Contact information: Zmanda Inc, 505 N Mathlida Ave, Suite 120
-# Sunnyvale, CA 94085, USA, or: http://www.zmanda.com
+# Contact information: Zmanda Inc, 465 S Mathlida Ave, Suite 300
+# Sunnyvale, CA 94086, USA, or: http://www.zmanda.com
 #
 
 
@@ -476,7 +476,7 @@ if ( $exit_value !=0 ) {
 }
 
 #create gnutar_list_dir
-if ( defined $tarlist && !defined $no_update_client ) {
+if ( defined $tarlist && !defined $no_client_update ) {
  system "$ssh", "$ssh_opt", "$amanda_user\@$client", "$mkdir", "$gnutar_list_dir";
  $exit_value  = $? >> 8;
 if ( $exit_value !=0 ) {
